@@ -60,6 +60,37 @@ def test_rechunk_internals_1():
     ]
     assert i1d[1] == answer4
 
+    new = cumdims_label(((1, 1, 2), (1, 5, 1, 0)), "n")
+    breaks = tuple(_breakpoints(o, n) for o, n in zip(old, new))
+    answer5 = (
+        ("o", 0),
+        ("n", 0),
+        ("o", 1),
+        ("n", 1),
+        ("o", 2),
+        ("o", 3),
+        ("o", 4),
+        ("o", 5),
+        ("n", 6),
+        ("n", 7),
+        ("n", 7),
+    )
+    assert breaks[1] == answer5
+    i1d = [_intersect_1d(b) for b in breaks]
+    answer6 = [
+        [(0, slice(0, 1))],
+        [
+            (1, slice(0, 1)),
+            (2, slice(0, 1)),
+            (3, slice(0, 1)),
+            (4, slice(0, 1)),
+            (5, slice(0, 1)),
+        ],
+        [(5, slice(1, 2))],
+        [(5, slice(2, 2))],
+    ]
+    assert i1d[1] == answer6
+
 
 def test_intersect_1():
     """Convert 1 D chunks"""
@@ -181,6 +212,10 @@ def test_rechunk_with_dict():
     y = x.rechunk(chunks={0: -1})
     assert y.chunks == ((24,), (8, 8, 8))
 
+    x = da.ones((24, 24), chunks=(4, 8))
+    y = x.rechunk(chunks={0: None, 1: "auto"})
+    assert y.chunks == ((4, 4, 4, 4, 4, 4), (24,))
+
 
 def test_rechunk_with_empty_input():
     x = da.ones((24, 24), chunks=(4, 8))
@@ -191,6 +226,10 @@ def test_rechunk_with_empty_input():
 def test_rechunk_with_null_dimensions():
     x = da.from_array(np.ones((24, 24)), chunks=(4, 8))
     assert x.rechunk(chunks=(None, 4)).chunks == da.ones((24, 24), chunks=(4, 4)).chunks
+    assert (
+        x.rechunk(chunks={0: None, 1: 4}).chunks
+        == da.ones((24, 24), chunks=(4, 4)).chunks
+    )
 
 
 def test_rechunk_with_integer():
@@ -905,3 +944,129 @@ def test_balance_split_into_n_chunks():
             x = da.from_array(np.random.uniform(size=N))
             y = x.rechunk(chunks=len(x) // nchunks, balance=True)
             assert len(y.chunks[0]) == nchunks
+
+
+def test_rechunk_with_zero():
+    a = da.ones((8, 8), chunks=(4, 4))
+    result = a.rechunk(((4, 4), (4, 0, 0, 4)))
+    expected = da.ones((8, 8), chunks=((4, 4), (4, 0, 0, 4)))
+
+    # reverse:
+    a, expected = expected, a
+    result = a.rechunk((4, 4))
+    assert_eq(result, expected)
+
+
+def test_intersect_chunks_with_nonzero():
+    from dask.array.rechunk import intersect_chunks
+
+    old = ((4, 4), (2,))
+    new = ((8,), (1, 1))
+    result = list(intersect_chunks(old, new))
+    expected = [
+        (
+            ((0, slice(0, 4, None)), (0, slice(0, 1, None))),
+            ((1, slice(0, 4, None)), (0, slice(0, 1, None))),
+        ),
+        (
+            ((0, slice(0, 4, None)), (0, slice(1, 2, None))),
+            ((1, slice(0, 4, None)), (0, slice(1, 2, None))),
+        ),
+    ]
+    assert result == expected
+
+
+def test_intersect_chunks_with_zero():
+    from dask.array.rechunk import intersect_chunks
+
+    old = ((4, 4), (2,))
+    new = ((4, 0, 0, 4), (1, 1))
+    result = list(intersect_chunks(old, new))
+
+    expected = [
+        (((0, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 4, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 4, None)), (0, slice(1, 2, None))),),
+    ]
+
+    assert result == expected
+
+    old = ((4, 0, 0, 4), (1, 1))
+    new = ((4, 4), (2,))
+    result = list(intersect_chunks(old, new))
+
+    expected = [
+        (
+            ((0, slice(0, 4, None)), (0, slice(0, 1, None))),
+            ((0, slice(0, 4, None)), (1, slice(0, 1, None))),
+        ),
+        (
+            ((3, slice(0, 4, None)), (0, slice(0, 1, None))),
+            ((3, slice(0, 4, None)), (1, slice(0, 1, None))),
+        ),
+    ]
+
+    assert result == expected
+
+    old = ((4, 4), (2,))
+    new = ((2, 0, 0, 2, 4), (1, 1))
+    result = list(intersect_chunks(old, new))
+    expected = [
+        (((0, slice(0, 2, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 2, None)), (0, slice(1, 2, None))),),
+        (((0, slice(2, 2, None)), (0, slice(0, 1, None))),),
+        (((0, slice(2, 2, None)), (0, slice(1, 2, None))),),
+        (((0, slice(2, 2, None)), (0, slice(0, 1, None))),),
+        (((0, slice(2, 2, None)), (0, slice(1, 2, None))),),
+        (((0, slice(2, 4, None)), (0, slice(0, 1, None))),),
+        (((0, slice(2, 4, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 4, None)), (0, slice(1, 2, None))),),
+    ]
+
+    assert result == expected
+
+    old = ((4, 4), (2,))
+    new = ((0, 0, 4, 4), (1, 1))
+    result = list(intersect_chunks(old, new))
+    expected = [
+        (((0, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((0, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((0, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 4, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 4, None)), (0, slice(1, 2, None))),),
+    ]
+
+    assert result == expected
+
+
+def test_old_to_new_with_zero():
+    from dask.array.rechunk import _old_to_new
+
+    old = ((4, 4),)
+    new = ((4, 0, 4),)
+    result = _old_to_new(old, new)
+    expected = [[[(0, slice(0, 4))], [(1, slice(0, 0))], [(1, slice(0, 4))]]]
+    assert result == expected
+
+    old = ((4,),)
+    new = ((4, 0),)
+    result = _old_to_new(old, new)
+    expected = [[[(0, slice(0, 4))], [(0, slice(4, 4))]]]
+    assert result == expected
+
+    old = ((4, 0, 4),)
+    new = ((4, 0, 2, 2),)
+    result = _old_to_new(old, new)
+    expected = [
+        [[(0, slice(0, 4))], [(2, slice(0, 0))], [(2, slice(0, 2))], [(2, slice(2, 4))]]
+    ]
+    assert result == expected
